@@ -10,10 +10,11 @@ Created on Sat Apr 11 06:41:28 2020
 """
 
 import os
+import shutil
 import json
 import logging
 import sqlite3
-from typing import List, Optional
+from typing import List, Dict, Optional
 from datetime import datetime as dt
 
 # Mapping des champs possibles d'un logging avec les type SQLite3
@@ -38,6 +39,7 @@ MAP_TYPE_HANDLER = dict(
     engine = 'TEXT'
 )
 
+
 class HandlerBase(logging.Handler):
     r"""
     Classe de base permettant de créer un handler à partir d'une liste de 
@@ -88,7 +90,7 @@ class HandlerBase(logging.Handler):
         La suite est composée des attributs de LogRecord créés
             engine : str
                 Nom du moteur ('sqlite', 'sqlalchemy', Warp10, etc.).
-    
+
     See also
     --------
     SQLiteHandler, SQLAlchemyHandler
@@ -142,6 +144,7 @@ class HandlerBase(logging.Handler):
             f"{self.__name__} qui hérite de la classe HandlerBase."
         )
 
+
 class SQLiteHandler(HandlerBase):
     r"""
     Classe permettant de créer un handler à partir d'une liste de Handler 
@@ -198,7 +201,7 @@ class SQLiteHandler(HandlerBase):
     :**kargs_sql: arguments d'entrée au connecteur 
         SQLite3 :
             sqlite3.connect(path_sql, **kargs_sql)
-    
+
     Examples
     --------
     Nous allons utiliser timer pour extraire les informations temporelles 
@@ -247,7 +250,7 @@ class SQLiteHandler(HandlerBase):
                                                  message
     0  {"name": "function_test", "dt": "0:00:00.500830"}
     1  {"name": "function_test", "dt": "0:00:00.500830"}
-    
+
     See also
     --------
     SQLAlchemyHandler, timer, timer_class
@@ -293,6 +296,7 @@ class SQLiteHandler(HandlerBase):
         conn.close()
         del conn
 
+
 class SQLAlchemyHandler(HandlerBase):
     r"""
     Classe permettant de créer un handler à partir d'une liste de Handler 
@@ -350,14 +354,14 @@ class SQLAlchemyHandler(HandlerBase):
     **kargs_sql :
         arguments d'entrée au connecteur SQLAlchemy :
             sqlalchemy.create_engine(path_sql, **kargs_sql)
-    
+
     Notes
     -----
     Cette classe nécessite SQLAlchemy, si ce package n'est pas présent dans 
     votre environnement il suffit d'ajouter le package "sqlalchemy" au fichier 
     environment.yml et de lancer le script bash bash_install_env.sh dans le 
     répertoire shell du projet.
-    
+
     Examples
     --------
     Nous allons utiliser timer pour extraire les informations temporelles 
@@ -407,7 +411,7 @@ class SQLAlchemyHandler(HandlerBase):
                                                  message
     0  {"name": "function_test", "dt": "0:00:00.500830"}
     1  {"name": "function_test", "dt": "0:00:00.500830"}
-    
+
     See also
     --------
     SQLiteHandler, timer, timer_class
@@ -468,17 +472,28 @@ class SQLAlchemyHandler(HandlerBase):
             )
         del engine
 
+
 class BaseArtimon(HandlerBase):
     r"""
     Classe de base permettant de gérer les données à envoyer dans Warp10 
     via Artimon. Cette classe n'est pas à utiliser directement, elle est 
     utiliser en tant que classe mère aux classes ArtimonAPIHandler et 
     ArtimonLocalHandler.
-    
+
     Parameters
     ----------
     producer : str
-        Nom du producer vers lequel les métriques seront envoyées.
+        Nom du producer vers lequel les métriques seront envoyées. Il doit 
+        avoir la forme <code_boite>.<nom_appli>
+    hostname : Optional[str]
+        nom facultatif du hostname (si non renseigné sera complété 
+        automatiquement par Artimon).
+    environment : Optional[str]
+        nom facultatif de l'environnement (si non renseigné sera complété 
+        automatiquement par Artimon).
+    label : Optional[dict]
+        label en complément de la métrique (comme l'environnement, la version,
+        etc.)
     format : List[str]
         liste des attributs du LogRecord qu'on souhaite récupérer et 
         stocker. Liste des attributs de LogRecord présentés à l'URL :
@@ -521,27 +536,24 @@ class BaseArtimon(HandlerBase):
         La suite est composée des attributs de LogRecord créés
             engine : str
                 Nom du moteur ('sqlite', 'sqlalchemy', 'Warp10', etc.).
-    hostname : Optional[str]
-        nom facultatif du hostname (si non renseigné sera complété 
-        automatiquement par Artimon).
-    environment : Optional[str]
-        nom facultatif de l'environnement (si non renseigné sera complété 
-        automatiquement par Artimon).
-    
+
     See also
     --------
     ArtimonAPIHandler, ArtimonLocalHendleur
     """
     def __init__(
-        self, producer: str,
-        format: List[str]=['name', 'msg'],
+        self,
+        producer: str,
         hostname: Optional[str]=None,
-        environment: Optional[str]=None
+        environment: Optional[str]=None,
+        label: Optional[Dict[str, str]]=None,
+        format: List[str]=['name', 'msg'],
     ) -> None:
         super().__init__(format = format)
         self.producer = producer
         self.hostname = hostname
         self.environment = environment
+        self.label = label
 
     def parser(self, record: logging.LogRecord) -> str:
         name = record.name
@@ -552,24 +564,21 @@ class BaseArtimon(HandlerBase):
             format.remove('name')
         if 'msg' in self.format:
             format.remove('msg')
-        log_att = record.__dict__
-        log_att = list(map(lambda x: log_att[x], format))
-        tags = {kk: vv for kk, vv in zip(format, log_att)}
+        tags = {kk: record.__dict__[kk] for kk in format}
         if self.hostname is not None:
             tags['hostname'] = self.hostname
         if self.environment is not None:
             tags['environment'] = self.environnement
         message = [
             dict(
-                i = dict(p=self.producer),
-                m = dict(
-                    n = name,
-                    t = tags
-                ),
-                v = record.msg,
-                c = dict(t=int(round(record.created*1000)))
+                i=dict(p=self.producer),
+                m=dict(n=name, t=tags),
+                v=record.msg,
+                c=dict(t=int(round(record.created*1000)))
             )
         ]
+        if self.label:
+            message[0]['l'] = self.label
         return json.dumps(message)
 
 class ArtimonAPIHandler(BaseArtimon):
@@ -586,6 +595,15 @@ class ArtimonAPIHandler(BaseArtimon):
         host de l'API Artimon (par défaut '127.0.0.1').
     port : str
         port de l'API Artimon (par défaut '9023').
+    hostname : Optional[str]
+        nom facultatif du hostname (si non renseigné sera complété 
+        automatiquement par Artimon).
+    environment : Optional[str]
+        nom facultatif de l'environnement (si non renseigné sera complété 
+        automatiquement par Artimon).
+    label : Optional[dict]
+        label en complément de la métrique (comme l'environnement, la version,
+        etc.)
     format : List[str]
         liste des attributs du LogRecord qu'on souhaite récupérer et stocker. 
         Liste des attributs de LogRecord présentés à l'URL :
@@ -628,13 +646,7 @@ class ArtimonAPIHandler(BaseArtimon):
         La suite est composée des attributs de LogRecord créés
             engine : str
                 Nom du moteur ('sqlite', 'sqlalchemy', 'Warp10', etc.).
-    hostname : Optional[str]
-        nom facultatif du hostname (si non renseigné sera complété 
-        automatiquement par Artimon).
-    environment : Optional[str]
-        nom facultatif de l'environnement (si non renseigné sera complété 
-        automatiquement par Artimon).
-    
+
     Notes
     -----
     Avec cette classe il n'est pas possible d'utiliser le format "message", 
@@ -690,9 +702,10 @@ class ArtimonAPIHandler(BaseArtimon):
         producer: str,
         host: str='127.0.0.1',
         port: str='9023',
-        format: List[str]=['name', 'msg'],
         hostname: Optional[str]=None,
-        environment: Optional[str]=None
+        environment: Optional[str]=None,
+        label: Optional[Dict[str, str]]=None,
+        format: List[str]=['name', 'msg']
     ) -> None:
         try:
             global requests
@@ -711,10 +724,11 @@ class ArtimonAPIHandler(BaseArtimon):
                 "utiliser plutôt l'attribut 'msg'."
             )
         super().__init__(
-            format = format,
-            producer = producer,
-            hostname = hostname,
-            environment = environment
+            format=format,
+            producer=producer,
+            hostname=hostname,
+            environment=environment,
+            label=label
         )
         self.host = host
         self.port = port
@@ -759,6 +773,19 @@ class ArtimonLocalHandler(BaseArtimon):
     path : str
         répertoire du dépot des métriques pour le collecteur Artimon (par 
         défaut '/var/run/artimonV2).
+    hostname : Optional[str]
+        nom facultatif du hostname (si non renseigné sera complété 
+        automatiquement par Artimon).
+    environment : Optional[str]
+        nom facultatif de l'environnement (si non renseigné sera complété 
+        automatiquement par Artimon).
+    label : Optional[dict]
+        label en complément de la métrique (comme l'environnement, la version,
+        etc.)
+    time_life : int (**deprecated**)
+        nombre de jour d'activité du fichier récoltant les métriques. Le
+        collecteur Artimon supprime automatiquement les fichiers qui ont 7 
+        jours d'inactivité (par défaut 1 jours).
     format : List[str]
         liste des attributs du LogRecord qu'on souhaite récupérer et stocker. 
         Liste des attributs de LogRecord présentés à l'URL :
@@ -801,17 +828,7 @@ class ArtimonLocalHandler(BaseArtimon):
         La suite est composée des attributs de LogRecord créés
             engine : str
                 Nom du moteur ('sqlite', 'sqlalchemy', 'Warp10', etc.).
-    hostname : Optional[str]
-        nom facultatif du hostname (si non renseigné sera complété 
-        automatiquement par Artimon).
-    environment : Optional[str]
-        nom facultatif de l'environnement (si non renseigné sera complété 
-        automatiquement par Artimon).
-    time_life : int (**deprecated**)
-        nombre de jour d'activité du fichier récoltant les métriques. Le
-        collecteur Artimon supprime automatiquement les fichiers qui ont 7 
-        jours d'inactivité (par défaut 1 jours).
-    
+
     Notes
     -----
     Avec cette classe il n'est pas possible d'utiliser le format "message", 
@@ -871,10 +888,10 @@ class ArtimonLocalHandler(BaseArtimon):
         self,
         producer: str,
         path: str='/var/run/artimonV2',
-        format: List[str]=['name', 'msg'],
         hostname: Optional[str]=None,
         environment: Optional[str]=None,
-        #time_life: int=1
+        label: Optional[Dict[str, str]]=None,
+        format: List[str]=['name', 'msg']
     ) -> None:
         if 'message' in format:
             raise AttributeError(
@@ -882,13 +899,13 @@ class ArtimonLocalHandler(BaseArtimon):
                 "utiliser plutôt l'attribut 'msg'."
             )
         super().__init__(
-            format = format,
-            producer = producer,
-            hostname = hostname,
-            environment = environment
+            format=format,
+            producer=producer,
+            hostname=hostname,
+            environment=environment,
+            label=label
         )
         self.path = path
-        # self.time_life = time_life
         if not os.path.exists(path):
             raise FileNotFoundError(
                 f"Le répertoire {path} n'existe pas."
@@ -897,30 +914,13 @@ class ArtimonLocalHandler(BaseArtimon):
     def emit(self, record: logging.LogRecord) -> None:
         self._asctime(record)
         self._engine(record, 'artimon')
-        file_name = f"{dt.now().strftime('%Y%m%d')}"
+        file_name = (
+            f"{self.path}{os.sep}{dt.now().strftime('%Y%m%d_%H:%M:%S:%f')}"
+        )
         with open(f"{file_name}.tmp", 'w') as file:
             file.write(self.parser(record)+'\n')
-        os.rename(f"{file_name}.tmp", f"{file_name}.artimon")
-        # A remove si plus necessaire
-        # file_name = sorted(
-        #     filter(
-        #         lambda x: x.endswith('.artimon'),
-        #         os.listdir()
-        #     ),
-        #     reverse = True
-        # )
-        # if len(file_name) > 0:
-        #     file_name = file_name[0]
-        #     delta_j = (
-        #         dt.now().date()
-        #         -dt.strptime(file_name.split('.')[0], '%Y%m%d').date()
-        #     ).days
-        #     if delta_j > self.time_life:
-        #         file_name = f"{dt.now().strftime('%Y%m%d')}.artimon"
-        # else:
-        #     file_name = f"{dt.now().strftime('%Y%m%d')}.artimon"
-        # with open(file_name, 'a') as file:
-        #     file.write(self.parser(record)+'\n')
+        shutil.move(f"{file_name}.tmp", f"{file_name}.artimon")
+
 
 class Warp10Handler(HandlerBase):
     r"""
@@ -985,6 +985,7 @@ class Warp10Handler(HandlerBase):
                 f"a renvoyer le code {response.status_code}."
             )
 
+
 class GTS:
     r"""
     Classe modélisant un objet GTS (Geo Time Series) Warp10.
